/* gqlist.h
 * Steven Rostedt.
 *
 * Since this is a wrapper for glist, and glist is GNU: 
 * I'm showing a GNU statement here!
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#ifndef __G_QLIST_H__
#define __G_QLIST_H__

/*
 * GQList is a wrapper for the GList. I originally 
 *  designed this in a separate project when I needed a queue.
 *  (hence the Q part in GQList :^)
 *  The glist was not efficient when handling a lot of additions
 *  to the end of the list.
 *  All I did was make an abstract object that would keep track
 *  of the last element in the list, so I would be able to 
 *  add to the end efficiently.
 *
 *  But like all things else, it grew!
 * 
 * Now GQList keeps track of the first, last and an arbitrary 
 *  "current" location that changes almost every time
 *   data is retrieved (but not removed).
 *
 * It allows indexing, using the three locations to find
 *  the closest route to get the indexed data.
 *
 * So traversing down the list using an index is still rather
 *  efficient!
 *
 * Out of the blue I decided to write a sort algorithm based on
 *  K&R's qsort algorithm in the ANSI C edition.
 *
 * I plan on writing many more interfaces for this object,
 *  but I appreciate the GNU environment I thought I would
 *  give this simple object over to anyone who likes to play ;)
 *
 *
 * !!!!!!!!! WARNING WARNING WARNING !!!!!!!!!!!
 *
 *  I HAVE NOT FULLY DEBUGGED THE ROUTINES, 
 *  Actually I did very few tests.
 *  So please use at YOUR OWN RISK!!!!
 *
 *  Please report bugs to me :) thank you in advance.
 *
 * S.D. Rostedt.
 *  steven.rostedt@lmco.com
 */

typedef struct _GQList GQList;

struct _GQList {
  GList *first;
  GList *last;
  GList *current;
  gint index;
  gint size;
};

/*
 * There are really four ways of initializing the gqlist object.
 *  gqlist = g_qlist_new(NULL);
 *  gqlist = g_qlist_new(some_g_list);
 *  gqlist = g_qlist_append(NULL,data);
 *  gqlist = g_qlist_prepend(NULL,data);
 */
GQList *
g_qlist_new(GList* glist);

/* 
 * This function simply adds an item to the end of the list.
 */
GQList *
g_qlist_append (GQList *gqlist, gpointer data);

/*
 * this function simply adds an item to the beginning of the list.
 */
GQList *
g_qlist_prepend (GQList *gqlist, gpointer data);

/*
 * This function inserts an item at some index. 
 * Note: index is used consistent with C. Starts with zero.
 * Insert z into location 2 of a list a,b,c returns
 *   a,b,z,c
 */
GQList *
g_qlist_insert_index (GQList *gqlist, gint indx, gpointer data);

/*
 * This function removes the first item from the list
 *  and returns that item (data)
 * NULL if list is empty.
 */
gpointer
g_qlist_remove_first (GQList *gqlist);

/*
 * This function removes the last item from the list
 *  and returns that item (data)
 * NULL if list is empty.
 */
gpointer
g_qlist_remove_last (GQList *gqlist);

/*
 * This function removes a item at a specified location on 
 *  the list. Note: index starts with zero.
 * NULL if bad index or list is empty.
 */
gpointer
g_qlist_remove_index (GQList *gqlist, gint indx);

/*
 * This function returns the first item on a list.
 * But does not modify the list. (Maybe I should declare
 *   gqlist as const GQList *)
 *  NULL if empty.
 */
gpointer 
g_qlist_get_first (GQList *gqlist);

/*
 * This function returns the last item on a list,
 * But does not modify the list.
 *  NULL if empty.
 */
gpointer 
g_qlist_get_last (GQList *gqlist);

/*
 * This function returns a data item a a specified location.
 *  NULL if bad location. 
 *  Does not modify the list.
 */
gpointer 
g_qlist_get_data (GQList *gqlist, gint indx);

/*
 * This function actually runs qsort on a link list :)
 */
void
g_qlist_sort (GQList *gqlist,
	      GCompareFunc func);


/******************** TBD TBD TBD TBD TBD TBD **********/
/*
 * The following are TBDs, 
 *  I just haven't had the time to write them yet.
 *  But I will (if someone else doesn't beat me to it :)
 */
gpointer
g_qlist_bsearch (GQList *gqlist,
		 GCompareFunc func,
		 gpointer data);

GQList *
g_qlist_insert_sorted (GQList *gqlist,
		    GCompareFunc func,
		    gpointer data);

gint
g_qlist_position(GQList *gqlist, gpointer data);

void
g_qlist_foreach (GQList *gqlist, 
		 GFunc func,
		 gpointer data);

GQList *
g_qlist_concat (GQList *gqlist1, GQList *gqlist2);


#endif
